// Functions for DIGITAL WATER GAUGE
// Version 4.0 by Nigel Mercier

#include <built_in.h>
#include "waterg.h"
#include "SH1106.h"
#include "functions.h"
#include "defs MK4 PCB.h"

byte getADC(byte adc)
{ // return 8-bit value from 10-bit ADC(adc), -FF /2  (ADC max range 727 to 275)
  delayX25ms(2) ;           // let reading stabilise after FET switch
  rawADC = ADC_Read(adc) ;  // get ADC reading, flag error if out of range
  if (rawADC > 768)
  { errFlag = 1 ; }
  if (rawADC < 256)
  { errFlag = 2 ; }
  return (rawADC - 0xFF) / 2 ; // munge 10-bit value to get in byte range
}

byte findNearA ( byte val )
{ // find index of closest value to ADC in SensorAft[]
  byte index ;
  signed int  delta, min = 255 ;
  for (i = 0; i < SizeAft; i++)  // iterate through from zero to size of table
  { // get absolute difference between SensorAft[i] and passed val
    delta = absVal((int)SensorAft[i] - (int)val) ;

    if (delta < min)
    {
      min = delta ;
      index = i ;
    }
  }
  return index;
}

byte findNearF ( byte val )
{ // find index of closest value to ADC in SensorFwd[]
  byte index ;
  signed int  delta, min = 255 ;
  for (i = 0; i < SizeFwd; i++)  // iterate through from zero to size of table
      { // get absolute difference between SensorFwd[i] and passed val
        delta = absVal((int)SensorFwd[i] - (int)val) ;
        // check if less than existing mimimum, if so update and save index
        if (delta < min)
        {
          min = delta ;
          index = i ;
        }
      }
   return index;
}

signed int absVal(signed int diff)
{ // simulate abs()
  if (diff < 0) // quicker and smaller than abs(), no library reqd
  {  diff = - diff ; } // munge abs value
  return diff ;
}

void beep(void)
{ // beep sounder for 100 milliseconds
  Sounder = 1 ;  // sounder ON
  delayX25ms(4) ;
  Sounder = 0 ;  // sounder OFF
}

void modeF(void) // FWD mode == 1
{ // use: ADC(1), FETf, findNearF, SensorFwd[], LitresFwd[]
  FETf = 1 ;                  // turn on FET for FWD sensor ground
  adc8 = getADC(1) ;          // get FWD sensor on ADC(1), also errFlag
  FETf = 0 ;                  // turn off FET for FWD sensor ground
  index = findNearF(adc8) ;   // get index of closest value in SensorFwd
  litVal = LitresFwd[index] ; // value at index (do *mult for display)
  // barG: percentage of fill for bargraph
  barG = (100*(float)litVal)/(float)LitresFwd[SizeFwd-1] ;
  strFont5XY("<", 0, 0) ;      // filled < FWD graphic top left
}

void modeA(void) // AFT mode == 2
{ // use: ADC(0), FETa, findNearA, SensorAft[], LitresAft[]
  FETa = 1 ;                  // turn on FET for AFT sensor ground
  adc8 = getADC(0) ;          // get AFT sensor on ADC(0), also errFlag
  FETa = 0 ;                  // turn off FET for AFT sensor ground
  index = findNearA(adc8) ;   // get index of closest value in SensorAft
  litVal = LitresAft[index] ; // value at index (do *mult for display)
  // barG: percentage of fill for bargraph
  barG = (100*(float)litVal)/(float)LitresAft[SizeAft-1] ;
  strFont5XY(">", (sWidth - 5), 0) ;   // filled > AFT graphic sWidth-5 top right
}

void displayLarge(int litValxMult)
{ // litValxMult is value from LitresAft or LitresFwd x 5
  byte xpos ;  // horizontal offset for centred display
  IntToStr(litValxMult, txt7) ; // convert integer to 7 char string ...
  Ltrim(txt7) ;                 // remove up to 5 leading spaces
  Rtrim(txt7) ;                 //   ... & 1 trailing space
  xpos = (sWidth / 2) - (strlen(txt7))*(34 / 2) ; // length 1, 2, 3 gives 47, 30, 13
  // litVal in litres in large 6-line font, centered by xpos
  strFont6RXY( (txt7), xpos , 0 ) ;
}

void delayX25ms(byte n)
{
  byte i ;
  for (i = 0 ; i < n ; i++)
    Delay_ms(25) ;
}

void helloWorld(void)
{
  while(1)
  {
  LED = 1 ;
  // PORTB = 0b00011110 ;
  Delay_ms(100);
  LED = 0 ;
  // PORTB = 0 ;
  Delay_ms(100);
  }
}