// Interrupt routines PIC16F1847
// DIGITAL WATER GAUGE
// Version 4.0 by Nigel Mercier www.mercier.org.uk

#include "waterg.h"

unsigned int intCount;    // Timer 2 interrupt counter

void configInt(void)
{
  T2CON =  0b00000110 ;    // Timer 2: ON, T2CLKPS1, T2CLKPS0 - Prescaler 16
  TMR2IF_bit = 0;          // clear TMR2IF in PIR1
  TMR2IE_bit = 1;          // enable Timer 2 interrupt in PIE1
  intCount = 0;            // initialize interrupt counter
  timeSec = 0;             //  and seconds counter
  INTCON = 0b11001000 ;    // set interupt enables GIE, PEIE, IOCE
  IOCBN =  0b11000000 ;    // interrupt on NEG change PORTB b6-b7 (switch inputs)
 }

void interrupt()
{ // ISR
  byte bTemp = PORTB ;     // snapshot of port B on entry, allows flags to be reset
  if (TMR2IF_bit)          // Timer 2 overflow?
  { // Fosc/4 = 2MHz (period 0.5uS), prescaler 16, interrupt @ 256 x 16 x 0.5
    intCount++ ;           // increment interrupt counter
    if (intCount >= (488)) // 256 x 16 x 0.5uS x 488 is about 1 second @ 8MHz
  { // do every second
    intCount = 0 ;         // reset interrupt counter
    timeSec++ ;            // increment seconds count
  }
  TMR2IF_bit = 0;          // clear TMR2IF
  }
  
  if (IOCBF & 0b11000000) // mask out lower bits as precaution
  { // PORTB changed, switch SWa or SWf pressed gives 0 on b6 or b7
    swMode = ((bTemp >> 6) ^ 0b00000011) ; // shift >> 6 to b1-b0, then toggle bits
    // swMode set to 0 (released or no switch), 1 (b7 SWa) or 2 (b6 SWf)
    if (swMode)      // key was pressed (not released, but not needed with NEG change flag)
    { 
      errFlag = 0 ;       // reset error flag for new reading
    }
    IOCBF = 0b00000000 ;  // reset all RB Port Change Interrupt Flags
  }
  
}